/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


#include <QVDesignerGUI>
#include <qvipp.h>
#include <QVApplication>
#include <QVMPlayerCamera>
#include <QVImageCanvas>
#include <QVNumericPlot>
#include <QVHistogramPlot>
#include <qvippworkers.h>

#ifndef DOXYGEN_IGNORE_THIS
class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name = QString()): QVWorker(name)  // el worker tiene que tener un constructor por defecto para poder registrarlo
			{
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Input image2", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Output image", outputFlag);
			addProperty<int>("Max pixel", outputFlag);
			addProperty<int>("Min pixel", outputFlag);
			addProperty<QList<double> >("MinMaxList", outputFlag);
			addProperty<QList<double> >("FirstRow", outputFlag);
			}

		void iterate()
			{
			QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			uchar min, max;
			Max(image, max);
			Min(image, min);
			setPropertyValue<int>("Max pixel", max);
			setPropertyValue<int>("Min pixel", min);
			timeFlag("get min-max values");

			QList<double> minmaxlist;
			minmaxlist << min << max;
			setPropertyValue<QList<double> >("MinMaxList", minmaxlist);
			timeFlag("get min-max list");

			QList<double> firstrow;
			for (uint i = 0; i < image.getCols(); i++) firstrow << image(i, 0);
			setPropertyValue<QList<double> >("FirstRow", firstrow);
			timeFlag("get first row");

			QVImage<uChar> dest(image);
			AddC(image, 10, dest);

			setPropertyValue< QVImage<uChar,1> >("Output image", dest);
			timeFlag("change image");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv, "Example program for QVision library." );
	qRegisterMetaType<MyWorker>("MyWorker"); // necesario para que se puedan crear desde el designerGUI
	
	MyWorker myWorker1("worker1");
	MyWorker myWorker2("worker2");
	QVIPPCopyP3C3Worker<uChar> p3c3("p3c3");
	QVMPlayerCamera camera("Video");
	camera.linkProperty(myWorker1, "Input image");
	myWorker1.linkProperty("Output image", myWorker2, "Input image", QVWorker::SynchronousLink);

	QVImageCanvas imageCanvas("image");
	myWorker2.linkProperty("Output image", imageCanvas);

	QVNumericPlot numericPlot("MinMax", false);
	myWorker1.linkProperty("Max pixel", numericPlot);
	myWorker1.linkProperty("Min pixel", numericPlot);

	QVDesignerGUI interface;

	return app.exec();
	}

#endif
	
