/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file plots.cpp
@ingroup ExamplePrograms
@brief Shows some image statistics and histograms in a set of plot widgets.

The plots displayed are graphs for the minimal and maximal pixel values for the input images, and histograms for the image.

@section Usage Usage of the program.
Once installed the QVision, compile using the <i>qmake</i> and <i>make</i> tools:
@code # qmake; make @endcode
Execute the application with the following line:
@code ./plots --URL=<source video identifier string> @endcode
For example:
@code ./plots --URL=http://perception.inf.um.es/public_data/videos/misc/minuto.avi @endcode
See section @ref QVMPlayerURLFormat for further info about the format and types of source video identifier strings.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvipp.h>

#include <QVApplication>
#include <QVMPlayerCamera>
#include <QVDefaultGUI>
#include <QVImageCanvas>
#include <QVNumericPlot>
#include <QVHistogramPlot>
#include <QVHistogramPlot2>

#ifndef DOXYGEN_IGNORE_THIS
class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Output image", outputFlag);
			addProperty<int>("Max pixel", outputFlag);
			addProperty<int>("Min pixel", outputFlag);
			addProperty<QList<double> >("MinMaxList", outputFlag);
			addProperty<QList<double> >("FirstRow", outputFlag);
			}

		void iterate()
			{
			QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");

			uchar min, max;
			Max(image, max);
			Min(image, min);
			setPropertyValue<int>("Max pixel", max);
			setPropertyValue<int>("Min pixel", min);
			QList<double> minmaxlist;
			minmaxlist << min << max;
			setPropertyValue<QList<double> >("MinMaxList", minmaxlist);

			QList<double> firstrow;
			for (uint i = 0; i < image.getCols(); i++) firstrow << image(i, 0);
			setPropertyValue<QList<double> >("FirstRow", firstrow);

			QVImage<uChar> dest(image);

			AddC(image, 10, dest);

			setPropertyValue< QVImage<uChar,1> >("Output image", dest);
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv, "Example program for QVision library." );
	
	MyWorker myWorker1("worker1");
	MyWorker myWorker2("worker2");
	MyWorker myWorker3("worker3");
	MyWorker myWorker4("worker4");
	myWorker1.pause();
	myWorker2.pause();
	myWorker3.pause();
	myWorker4.pause();

	QVMPlayerCamera camera1("Video1");
	QVMPlayerCamera camera2("Video2");

	camera1.linkProperty(&myWorker1, "Input image", QVWorker::AsynchronousLink);
	camera2.linkProperty(&myWorker2, "Input image", QVWorker::AsynchronousLink);

	myWorker1.linkProperty("Output image", myWorker2, "Input image", QVWorker::SequentialLink);
	myWorker3.linkProperty("Output image", myWorker4, "Input image", QVWorker::SequentialLink);
	myWorker2.linkProperty("Output image", myWorker3, "Input image", QVWorker::SequentialLink);

	QVDefaultGUI interface;

	QVImageCanvas imageCanvas1("image1");
	QVImageCanvas imageCanvas2("image2");
	myWorker3.linkProperty("Output image", imageCanvas1);
	myWorker4.linkProperty("Output image", imageCanvas2);
/*
	QVNumericPlot numericPlot("MinMax", false);
	myWorker1.linkProperty("Max pixel", numericPlot);
	myWorker1.linkProperty("Min pixel", &numericPlot);

	QVHistogramPlot hist("hist");
	myWorker1.linkProperty("FirstRow", hist);
	myWorker2.linkProperty("FirstRow", hist);
// 	myWorker4.linkProperty("MinMaxList", hist);

	QVHistogramPlot2 hist2("hist2");
	myWorker1.linkProperty("FirstRow", hist2);
	myWorker2.linkProperty("FirstRow", hist2);
// 	myWorker3.linkProperty("MinMaxList", hist2);
*/
	return app.exec();
	}

#endif
	
