/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file videoPlayer.cpp
@ingroup ExamplePrograms
@brief Simple example of image input/output from a video file, to a canvas window

This program will display in an image canvas the contents of a video file or the input from a video resource.

@section Usage Usage of the program.
Once installed the QVision, compile using the <i>qmake</i> and <i>make</i> tools:
@code # qmake; make @endcode
Execute the application with the following line:
@code ./videoPlayer --URL=<source video identifier string> @endcode
For example:
@code ./videoPlayer --URL=http://perception.inf.um.es/public_data/videos/misc/minuto.avi @endcode
See section @ref QVMPlayerURLFormat for further info about the format and types of source video identifier strings.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <QVApplication>
#include <QVMPlayerCamera>
#include <QVDefaultGUI>
#include <QVImageCanvas>
#include <QVYUV4MPEG2CameraWorker>

#ifndef DOXYGEN_IGNORE_THIS
class PlayerWorker: public QVWorker
	{
	public:
		PlayerWorker(QString name): QVWorker(name)
			{
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			}

		void iterate()	{ }
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv, "Example program for QVision library. Displays the contents of a video source.");

	QVYUV4MPEG2CameraWorker camera("YUV Camera");
	//QVMPlayerCamera camera("Video");

	PlayerWorker player("Video player");
	//camera.linkProperty(&player,"Input image", QVWorker::AsynchronousLink);
	camera.linkProperty(&player,"Input image", QVWorker::SynchronousLink);

	QVDefaultGUI interface;

	QVImageCanvas imageCanvas("Output image");
	player.linkProperty("Input image", imageCanvas);

	return app.exec();
	}

#endif

