/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef DESIGNERGUI_H
#define DESIGNERGUI_H

#include <QObject>
#include <QList>
#include <QString>
#include <QMap>
#include "qvgui/qvgui.h"
#include "facade/itemproperties.h"
#include "facade/itemfactory.h"
#include "slate/slatewindow.h"
#include "slate/informer.h"

#ifndef DOXYGEN_IGNORE_THIS

class QVPropertyContainer;
class QVPropertyContainerChange;

// Clase que se encargará de hacer de intermediario entre la vista y el modelo (el slate y la QVAplication)
class QVDesignerGUI: QObject, QVGUI
{
	Q_OBJECT
	
	public:
		QVDesignerGUI();

		//////////////////////////////////////////// funcionalidad de gui de la qvision ////////////////////////////////////////////

		/// @brief initialice the QVGUI object.
		virtual void init();
	
		/// @brief show the QVGUI object.
		void show();


		//////////////////////////////////////////// obtención de los items disponibles ////////////////////////////////////////////

		// devuelve la lista de Items disponibles en el sistema, cada uno de estos items es un elemento del diseño de los programas de QVision
		QList<QString> getItemTypes() const;
	
		// devuelve la lista de Items de entrada disponibles en el sistema, cada uno de estos items es un elemento del diseño de los programas de QVision
		QList<QString> getInputItemTypes() const;
	
		// devuelve la lista de Items intermedios disponibles en el sistema, cada uno de estos items es un elemento del diseño de los programas de QVision
		QList<QString> getMiddleItemTypes() const;
	
		// devuelve la lista de Items de salida disponibles en el sistema, cada uno de estos items es un elemento del diseño de los programas de QVision
		QList<QString> getOutputItemTypes() const;

		QString getCppText() const;

		QString getXMLText() const;


		////////////////////////////////////// modificación del entorno común (slate + qvision) //////////////////////////////////////

		// crea y registra un item, del tipo especificado y nombre concreto, en el sistema
		bool addItem(const QString type, const QString name);
	
		// crea y registra un link entre dos items del sistema
		bool addLink(const QString fromName, const QString fromProp, const QString toName, const QString toProp, const bool synchronous, const bool sequential);

		// borra y desregistra un item
		bool delItem(const QString name);
	
		// borra y desregistra un link
		bool delLink(const QString fromName, const QString fromProp, const QString toName, const QString toProp);

		// establece el valor de la propiedad de un item del sistema
		template <class Type> bool setProperty(const QString fromName, const QString fromProp, const Type &value);

		void showProperties(const QString itemName);

		///////////////////////////////////////////////////// control del entorno /////////////////////////////////////////////////////

		// lanza un sistema como el compuesto en la pizarra
		void run();
	
		// para el sistema lanzado
		void stop();

	public slots:
		void quit();
		void processChange(QVPropertyContainerChange change);
		void dialogChange(QVPropertyContainerChange change);

	private:
		bool createDialog(const QString itemName);
		void deleteDialog(const QString itemName);
		QString getLinkName(QVPropertyContainerChange change);
		QString getLinkName(QString fromName, QString fromProp, QString toName, QString toProp);
		QString getAbsPropName(QString fromName, QString fromProp);

		ItemFactory factory;
		SlateWindow slate;
		Informer informer;

		QMap<QString, QVPropertyContainer *> containers;
		QMap<QString, QVPropertyContainer *> initialContainers;
		QMap<QString, QDialog *> dialogs;

			class CreatedLink
				{
				public:
					CreatedLink(const QString _fromName, const QString _fromProp, const QString _toName, const QString _toProp, const bool _sync, const bool _sequ):
						fromName(_fromName), fromProp(_fromProp), toName(_toName), toProp(_toProp), synchronous(_sync), sequential(_sequ) { }

					QString fromName, fromProp, toName, toProp;
					bool synchronous;
					bool sequential;
				};

			class CreatedItem
				{
				public:
					CreatedItem(const QString _type, const QString _name): type(_type), name(_name) { }

					QString type, name;
				};

			class PropertyChange
				{
				public:
					PropertyChange(const QString cont, const QString prop, const QVariant val): contName(cont), propName(prop), value(val) { }

					QString contName, propName;
					QVariant value;
				};

		QMap<QString, CreatedLink> createdLinks; // the key is "fromName[fromProp] => toName[toProp]"
		QMap<QString, CreatedItem> createdItems;
		QMap<QString, PropertyChange> propertyChanges; // the key is "contName[propName]"
};

#endif

#endif

