/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


#include <QtGui>

#include "groupnode.h"
#include <QGraphicsSceneDragDropEvent>
#include "itemnode.h"


GroupNode::GroupNode(QGraphicsItem * parent, QGraphicsScene * scene): Node(parent, scene)
{
    myTextColor = Qt::darkGreen;
    myOutlineColor = Qt::darkBlue;
    myBackgroundColor = Qt::white;

    setFlags(ItemIsMovable | ItemIsSelectable);

	type = "Group";
	name = type;
}

GroupNode::~GroupNode()
{
     foreach (Link *link, getLinks())
        delete link;

    foreach (QGraphicsItem *child, children())
        delete child;
}

void GroupNode::addNode(Node *node)
{
	//se actualiza cambio el padre
	node->setParentItem(this);

	//se escala
	node->scale(SUBSCALE, SUBSCALE);

	//se escala su posición para que no se separen cuando se unen
	node->setPos(node->pos() * SUBSCALE);
}

void GroupNode::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget * /* widget */)
{
    QPen pen(myOutlineColor);
    if (option->state & QStyle::State_Selected) {
        pen.setStyle(Qt::DotLine);
        pen.setWidth(2);
    }
    painter->setPen(pen);
    painter->setBrush(myBackgroundColor);

    QRectF rect = outlineRect();
    painter->drawRoundRect(rect.adjusted(0.0, 12.0, 0.0, .0), roundness(rect.width()),
                           roundness(rect.height()));

    painter->setPen(myTextColor);
    painter->drawText(QPointF((int)rect.left(), (int)rect.top() + 10), myText);


	painter->drawEllipse((int)rect.right() - 10, (int)rect.bottom() - 10, 8, 8);
    // una forma de mostrar cosas solo cuand tenemos un nivel de detalle concreto, esto se puede poner en función del nivel de anidamiento en que se encuentra.
    // hacer uno recursivo que tenga dentro otro objeto igual con nivel de anidamiento padre+1.
//     if (option->levelOfDetail > 2)
//     {
// 	QStyleOptionGraphicsItem *nodesoption = new QStyleOptionGraphicsItem(*option);
// // 	nodesoption->levelOfDetail = option->levelOfDetail / 2;
// // 	nodesoption->matrix = nodesoption->matrix.scale(0.5, 0.5);
//         foreach (Node *node, myNodes)
//             node->paint(painter, nodesoption, 0);
//     }
}

// devuelve la posición (en cordenadas de la scena) del centro del punto indicado por el índice point, si se sale del rango devuelve la posición del nodo
QPointF GroupNode::scenePointPos(int ) const
{
/*	QRectF rect = outlineRect();

	if (point < 0) return this->scenePos();
	if (point < numProp) return mapToScene( QPointF(rect.left() + 8, rect.top() + 16*point + 24) );
	if (point < 2 * numProp) return mapToScene( QPointF(rect.right() - 8, rect.top() + 16*(point-numProp) + 24) );*/
	return this->scenePos();
}

QPointF GroupNode::scenePointPos(QString, bool) const
{
	return this->scenePos();
}

int GroupNode::propPoint(QString, bool) const
{
	return 0;
}

int GroupNode::numProps() const
{
	return 0;
}

void GroupNode::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
{
    QRectF rect = outlineRect();
	QPointF click = mapFromScene(event->scenePos());

	if ((click.x() > rect.right() - 10) && (click.y() > rect.bottom() - 10)) {
		// aqui es donde tendría que cambiar de la vista de nodos unidos a los puntos laterales a la vista de los nombres de las
		// propiedades de entrada y salida.
	}
	else {
		QString text = QInputDialog::getText(event->widget(),
							tr("Edit Text"), tr("Enter new text:"),
							QLineEdit::Normal, myText);
		if (!text.isEmpty())
			setText(text);
	}
}

QVariant GroupNode::itemChange(GraphicsItemChange change,
                          const QVariant &value)
{
    if (change == ItemPositionHasChanged)
		updateLinksPos();

    return QGraphicsItem::itemChange(change, value);
}

void GroupNode::updateLinksPos()
{
    foreach (Link *link, getLinks())
        link->trackNodes();

	foreach (QGraphicsItem *child, children()) {
		Node *nodeChild = dynamic_cast<Node *>(child);
		if (nodeChild)
			nodeChild->updateLinksPos();
	}
}

QRectF GroupNode::outlineRect() const
{
    const int Padding = 8;
    QFontMetricsF metrics = qApp->font();
    QRectF rect = metrics.boundingRect(myText);
    rect.adjust(-Padding, -Padding, +Padding, +Padding);
    rect.translate(-rect.center());

    QRectF childRect = childrenBoundingRect();

    //return (childRect.unite(rect)).adjusted(-5.0, -1.0, 5.0, 1.0);
	//quitando la union es como si se desplazara el rectángulo sin que la linea de los link se diera cuenta, quedando mál
	return (childRect).adjusted(-20.0, -17.0, 20.0, 5.0);
}

int GroupNode::roundness(double size) const
{
    const int Diameter = 12;
    return 100 * Diameter / int(size);
}

int GroupNode::pointAt(QPointF ) const
{
	return -1;
}

QString GroupNode::propName(int ) const
{
	return QString();
}

