/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SLATEWINDOW_H
#define SLATEWINDOW_H

#include <QMainWindow>
#include <QCloseEvent>
#include <QPair>
#include <QAction>


class QGraphicsItem;
class QGraphicsScene;
class QGraphicsView;
class Link;
class Node;
class ItemNode;
class ItemProperties;
class SlateView;
class QVDesignerGUI;

#ifndef DOXYGEN_IGNORE_THIS

// usar QGraphicsItem::collidesWithItem() para meter automátcamente un item dentro de un grupo (debería estar en node)
class SlateWindow : public QMainWindow
{
    Q_OBJECT

public:
    SlateWindow(QVDesignerGUI *desig, QWidget * parent = 0);
	bool createLink(Node *fromNode, int fromPoint, Node *toNode, int toPoint);
	void addLinkLine(QString fromNode, QString fromProp, QString toNode, QString toProp, bool sinc, bool sequential);
	void delLinkLine(QString fromNode, QString fromProp, QString toNode, QString toProp);
	void addItemNode(QString type, QString name, ItemProperties *item);
	void delItemNode(QString name);
	void addProperty(QString srcName, QString propName, int type, bool in, bool out);
	void delProperty(QString srcName, QString propName);
	void closeEvent(QCloseEvent *event);
	void includeItemType(QString itemType);
	void arrangeItems();

public slots:
    void showProperties();
	void showProperties(Node *node);
	void showMessage(QString message);

private slots:
    void del();
	void join();
    void bringToFront();
    void sendToBack();
    void updateActions();
	void insertItem(QString type);
	void run();
	void stop();
	bool saveAs();

signals:
	/// @brief Emited when the window closes.
	void closed();

private:
    typedef QPair<Node *, Node *> NodePair;

    QPoint startPos;

    void createMenus();
    void createToolBars();
    void setZValue(int z);
    void setupNode(Node *node);
    Node *selectedNode() const;
    Link *selectedLink() const;
    NodePair selectedNodePair() const;
    QList<QGraphicsItem *> selectedNodeGroup() const;
	QList<QGraphicsItem *> onlyParents(QList<QGraphicsItem *> items) const;
    bool saveFile(const QString &fileName);
	bool writeCppFile(const QString &fileName);
	int precursors(ItemNode *node);

    QMenu *fileMenu;
    QMenu *editMenu;
	QMenu *insertMenu;
    QToolBar *editToolBar;
    QAction *exitAction;
    QAction *addSLinkAction;
    QAction *addALinkAction;
    QAction *addQLinkAction;
	QActionGroup *linkGroup;
    QAction *joinAction;
    QAction *deleteAction;
    QAction *bringToFrontAction;
    QAction *sendToBackAction;
    QAction *propertiesAction;
	QAction *runAction;
	QAction *stopAction;
	QAction *saveAsAction;

	QStatusBar *statusbar;

    QGraphicsScene *scene;
    SlateView *view;
	QVDesignerGUI *designer;

    int minZ;
    int maxZ;
    int seqNumber;

	QMap<QString, ItemNode *> insertNodes;
	QMap<QString, QPointF> insertNodesPos;
	QMap<QString, Link *> insertLinks;
};

#endif
#endif
