/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvimagecanvas.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVIMAGECANVAS_H
#define QVIMAGECANVAS_H

#include <QPointF>
#include <QVWorker>
#include <QVPolyline>
#include <QVPolylineF>
#include <qvgui/qvcanvas.h>
#include <qvgui/qvplot.h>

/*!
@class QVImageCanvas qvimagecanvas.h qvgui/qvimagecanvas.h
@brief Viewer widget for @ref QVImage objects and other data types.

Displays images from an image source:

@image html qvimagecanvas_penguin.png

For further information about its usage, see section @ref QVImageCanvasWidget.
@ingroup qvgui
*/

class QVImageCanvas: public QVCanvas, public QVPropertyContainer
	{
		Q_OBJECT
	public:
		QVImageCanvas(const QString name = QString(), QWidget *parent=0);

		/*bool linkProperty(QString sourcePropertyName, QVPropertyContainer *destinyContainer, QString destinyPropertyName, LinkType linkType = AsynchronousLink);
		bool linkProperty(QString sourcePropertyName, QVPropertyContainer &destinyContainer, QString destinyPropertyName, LinkType linkType = AsynchronousLink);
		bool linkProperty(QVPropertyContainer *destinyContainer, QString destinyPropertyName, LinkType linkType = AsynchronousLink);
		bool linkProperty(QVPropertyContainer &destinyContainer, QString destinyPropertyName, LinkType linkType = AsynchronousLink);
		bool linkProperty(QString sourcePropertyName, QVPropertyContainer *destinyContainer, LinkType linkType = AsynchronousLink);
		bool linkProperty(QString sourcePropertyName, QVPropertyContainer &destinyContainer, LinkType linkType = AsynchronousLink);
		void linkProperty(QVPropertyContainer *container, LinkType linkType = AsynchronousLink);
		void linkProperty(QVPropertyContainer &container, LinkType linkType = AsynchronousLink);
		bool unlinkProperty(QString origName, QVPropertyContainer *destCont, QString destName);
		bool unlinkProperty(QString origName, QVPropertyContainer &destCont, QString destName);*/
		void unlink();

		void setLowHigh(float low,float high);
		#ifndef DOXYGEN_IGNORE_THIS
		void viewer();
		#endif
		virtual void custom_viewer() { };

		bool setDisplayColor(const QString &name, const QColor &color)	{ return setPropertyValue<QColor>("Color for " + name, color); }
		bool setPrintTags(const QString &name, const bool &printTags)	{ return setPropertyValue<bool>("Print tags for object" + name, printTags); }
		bool setRadiusSize(const QString &name, const int &radius)	{ return setPropertyValue<int>("Radius for points in " + name, radius); }

	public slots:
		void rectSelectedSlot(QRect rect);
		void polySelectedSlot(QPoint point, bool reset, TPolyMode mode);
		void circleSelectedSlot(QPoint center, float radius);

	protected:
		bool linkUnspecifiedInputProperty(QVPropertyContainer *sourceContainer, QString sourcePropName, LinkType linkType = AsynchronousLink);
		bool linkUnspecifiedOutputProperty(QVPropertyContainer *destContainer, QString destPropName, LinkType linkType = AsynchronousLink);
		bool treatUnlinkInputProperty(QString destPropName, QVPropertyContainer *sourceCont, QString sourcePropName);

		void draw(const QList<QPoint> &pointList, QColor color = Qt::red, bool printTags = false, int radius = 3);
		void draw(const QList<QPointF> &pointList, QColor color = Qt::red, bool printTags = false, int radius = 3);
		void draw(const QVPolyline &polyline, QColor color = Qt::red, bool printTags = false);
		void draw(const QVPolylineF &polylinef, QColor color = Qt::red, bool printTags = false);
		void draw(const QRect &rectangle, QColor color = Qt::red, bool printTags = false);

	private:

		const QColor	getNextColor()
			{
			QColor color = qvColors[colorCursor++];
			colorCursor %= 10;

			return color;
			}

		float _low,_high;
		int colorCursor, contentLinkedWorkers;
	};

Q_DECLARE_METATYPE(TPolyMode);

#endif
