/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvmplayercameraworker.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <QVMPlayerCameraWorker>
#include <qvipp.h>

#include <iostream>
#include <qvdefines.h>

QVMPlayerCameraWorker::QVMPlayerCameraWorker(QString name) : QVCameraWorker(name)
{
	addProperty<bool>("Deinterlaced", inputFlag, FALSE,"If the camera should be opened in deinterlaced mode");
}


bool QVMPlayerCameraWorker::openCam(QString fileName, int &cols, int &rows, int &fps)
	{
	// Try to open mplayerreader, and read correct rows, cols and fps.

	QVMPlayerReader::OpenOptions options = QVMPlayerReader::Default;
	if(getPropertyValue<bool>("RealTime")) options |= QVMPlayerReader::RealTime;
	if(getPropertyValue<bool>("Deinterlaced")) options |= QVMPlayerReader::Deinterlaced;
	if(getPropertyValue<bool>("NoLoop")) options |= QVMPlayerReader::NoLoop;

	if(not mplayerreader.openCam(fileName,options,cols,rows))
		{
		QString msg = QString("MPlayer can not open ") + fileName ;
		qWarning() << msg;
		setLastError(msg);
		return FALSE;
		}
	else	// Everything correct.
		{
		cols = mplayerreader.getCols();
		rows = mplayerreader.getRows();
		fps = mplayerreader.getFPS();

		if(mplayerreader.isLiveCamera() and not getPropertyValue<bool>("RealTime"))
			qWarning() << 	"QVMPlayerCameraWorker::openCam: Warning, live cameras should always be opened in\n"
					"RealTime mode; otherwise, strange delayed (buffered) frames behaviour could be\n"
					"observed when connected to slow workers, or when pausing the camera.\n";

		return TRUE;
		}
	}

void QVMPlayerCameraWorker::closeCam()
	{
	mplayerreader.closeCam();
	}

bool QVMPlayerCameraWorker::grab(QVImage<uChar,1> &imgY, QVImage<uChar,1> &imgU, QVImage<uChar,1> &imgV)
	{
	return mplayerreader.grab(imgY, imgU, imgV);
	}










