/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvippworkers.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvippworkers.h>
#include <qvip/qvipp/macros.h>

////////////////////////////////////
// Macros
// IppiMaskSize
#define ADD_IPPI_MASK_SIZE_PROPERTY(Name)		\
	QVIndexedStringList Name ## _choice;		\
	/*Name ## _choice.append("ippMskSize1x3");	\
	Name ## _choice.append("ippMskSize1x5");	\
	Name ## _choice.append("ippMskSize3x1");*/	\
	Name ## _choice.append("ippMskSize3x3");	\
	/*Name ## _choice.append("ippMskSize5x1");*/	\
	Name ## _choice.append("ippMskSize5x5");	\
	addProperty< QVIndexedStringList >("" # Name, inputFlag, Name ## _choice);

#define	GET_IPPI_MASK_SIZE_PROPERTY(Name)								\
	const QString Name ## _indexedStringList =							\
				getPropertyValue< QVIndexedStringList >("" #Name).getCurrent();		\
	const IppiMaskSize Name =									\
				(Name ## _indexedStringList == "ippMskSize1x3")? ippMskSize1x3:		\
				(Name ## _indexedStringList == "ippMskSize1x5")? ippMskSize1x5:		\
				(Name ## _indexedStringList == "ippMskSize3x1")? ippMskSize3x1:		\
				(Name ## _indexedStringList == "ippMskSize3x3")? ippMskSize3x3:		\
				(Name ## _indexedStringList == "ippMskSize5x1")? ippMskSize5x1:		\
				ippMskSize5x5;

// IppCmpOp
#define ADD_IPP_CMP_OP_PROPERTY(Name)			\
	QVIndexedStringList Name ## _choice;		\
	Name ## _choice.append("ippCmpLessEq");		\
	Name ## _choice.append("ippCmpEq");		\
	Name ## _choice.append("ippCmpGreaterEq");	\
	Name ## _choice.append("ippCmpGreater");	\
	addProperty< QVIndexedStringList >("" # Name, inputFlag, Name ## _choice);

#define	GET_IPP_CMP_OP_PROPERTY(Name)									\
	const QString Name ## _indexedStringList =							\
				getPropertyValue< QVIndexedStringList >("" #Name).getCurrent();		\
	const IppCmpOp Name =									\
				(Name ## _indexedStringList == "ippCmpLessEq")? ippCmpLessEq:		\
				(Name ## _indexedStringList == "ippCmpEq")? ippCmpEq:			\
				(Name ## _indexedStringList == "ippCmpGreaterEq")? ippCmpGreaterEq:	\
				ippCmpGreater;

#define	CREATE_OUTPUT_IMAGE(Type, Channels, Dest, Src, DestROIOffset)	\
	QVImage<Type, Channels> Dest(Src.getROI().width() + DestROIOffset.x(), Src.getROI().height() + DestROIOffset.y());

////////////////////////////////////
// QVIPPCopyP3C3Worker<uChar,1>
template <> QVIPPCopyP3C3Worker<uChar>::QVIPPCopyP3C3Worker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src1", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("src2", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("src3", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPCopyP3C3Worker<uChar>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1>	src1 = getPropertyValue< QVImage<uChar,1> >("src1"),
				src2 = getPropertyValue< QVImage<uChar,1> >("src2"),
				src3 = getPropertyValue< QVImage<uChar,1> >("src3");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	//CHECK_SAME_ROISIZE(src1, src2);
	//CHECK_SAME_ROISIZE(src2, src3);

	// Apply dilate 3x3 filter	
	CREATE_OUTPUT_IMAGE(uChar, 3, dest, src1, destROIOffset);
	Copy(src1, src2, src3, dest, destROIOffset);

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}


// QVIPPCopyC3P3Worker<uChar,1>
template <> QVIPPCopyC3P3Worker<uChar>::QVIPPCopyC3P3Worker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);

	addProperty< QVImage<uChar,1> >("dest1", outputFlag);
	addProperty< QVImage<uChar,1> >("dest2", outputFlag);
	addProperty< QVImage<uChar,1> >("dest3", outputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPCopyC3P3Worker<uChar>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3>	src = getPropertyValue< QVImage<uChar,3> >("src");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Apply dilate 3x3 filter	
	CREATE_OUTPUT_IMAGE(uChar, 1, dest1, src, destROIOffset);
	CREATE_OUTPUT_IMAGE(uChar, 1, dest2, src, destROIOffset);
	CREATE_OUTPUT_IMAGE(uChar, 1, dest3, src, destROIOffset);
	Copy(src, dest1, dest2, dest3, destROIOffset);

	setPropertyValue< QVImage<uChar,1> >("dest1", dest1);
	setPropertyValue< QVImage<uChar,1> >("dest2", dest2);
	setPropertyValue< QVImage<uChar,1> >("dest3", dest3);
	timeFlag("Write output parameters");
	}

// QVIPPDilate3x3Worker<uChar,1>
template <> QVIPPDilate3x3Worker<uChar,1>::QVIPPDilate3x3Worker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPDilate3x3Worker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Apply dilate 3x3 filter	
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);
	//QVImage<uChar,1> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());
	Dilate3x3(src, dest);

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPErode3x3Worker<uChar,1>
template <> QVIPPErode3x3Worker<uChar,1>::QVIPPErode3x3Worker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPErode3x3Worker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Apply dilate 3x3 filter	
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);
	Erode3x3(src, dest);

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}


// QVIPPComputeThreshold_OtsuWorker<uChar,1>
template <> QVIPPComputeThreshold_OtsuWorker<uChar,1>::QVIPPComputeThreshold_OtsuWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);

	addProperty< int >("Threshold Otsu", outputFlag);
	}

template <> void QVIPPComputeThreshold_OtsuWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	timeFlag("Read input parameters");

	// Apply random filter	
	uChar threshold;
	ComputeThreshold_Otsu(src, threshold);

	setPropertyValue< int >("Threshold Otsu", threshold);
	timeFlag("Write output parameters");
	}


// QVIPPAddRandUniform_DirectWorker<uChar,1>
template <> QVIPPAddRandUniform_DirectWorker<uChar,1>::QVIPPAddRandUniform_DirectWorker(QString name): QVWorker(name), seed(0)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Low", inputFlag, 10, "", 0, 255);
	addProperty< int >("High", inputFlag, 30, "", 0, 255);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPAddRandUniform_DirectWorker<uChar,1>::iterate()
	{
	// Read input properties
	QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const uChar	low = getPropertyValue< uChar >("Low"),
			high = getPropertyValue< uChar >("High");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Apply random filter	
	AddRandUniform_Direct(src, low, high, seed);

	seed++;

	setPropertyValue< QVImage<uChar,1> >("dest", src);
	timeFlag("Write output parameters");
	}

// QVIPPFilterBoxWorker<uChar,1>
template <> QVIPPFilterBoxWorker<uChar,1>::QVIPPFilterBoxWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterBoxWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterBox(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterBox_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterBoxWorker<uChar,3>
template <> QVIPPFilterBoxWorker<uChar,3>::QVIPPFilterBoxWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterBoxWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterBox(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterBox_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}


// QVIPPFilterMedianWorker<uChar,1>
template <> QVIPPFilterMedianWorker<uChar,1>::QVIPPFilterMedianWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterMedianWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterMedian(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMedian_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterMedianWorker<uChar,3>
template <> QVIPPFilterMedianWorker<uChar,3>::QVIPPFilterMedianWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterMedianWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterMedian(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMedian_8u_C3R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterMaxWorker<uChar,1>
template <> QVIPPFilterMaxWorker<uChar,1>::QVIPPFilterMaxWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterMaxWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterMax(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMax_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterMaxWorker<uChar,3>
template <> QVIPPFilterMaxWorker<uChar,3>::QVIPPFilterMaxWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterMaxWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterMax(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMax_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterMinWorker<uChar,1>
template <> QVIPPFilterMinWorker<uChar,1>::QVIPPFilterMinWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterMinWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterMin(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMin_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterMinWorker<uChar,3>
template <> QVIPPFilterMinWorker<uChar,3>::QVIPPFilterMinWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterMinWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterMin(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMin_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterGaussWorker<uChar,1>
template <> QVIPPFilterGaussWorker<uChar,1>::QVIPPFilterGaussWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	ADD_IPPI_MASK_SIZE_PROPERTY(maskSize);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterGaussWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	GET_IPPI_MASK_SIZE_PROPERTY(maskSize);
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterGauss(src, dest, maskSize, destROIOffset);
	timeFlag("Call to ippiFilterGauss_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterHipassWorker<uChar,1>
template <> QVIPPFilterHipassWorker<uChar,1>::QVIPPFilterHipassWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	ADD_IPPI_MASK_SIZE_PROPERTY(maskSize);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterHipassWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	GET_IPPI_MASK_SIZE_PROPERTY(maskSize);
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterHipass(src, dest, maskSize, destROIOffset);
	timeFlag("Call to ippiFilterHipass_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterGaussWorker<uChar,3>
template <> QVIPPFilterGaussWorker<uChar,3>::QVIPPFilterGaussWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	ADD_IPPI_MASK_SIZE_PROPERTY(maskSize);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterGaussWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	GET_IPPI_MASK_SIZE_PROPERTY(maskSize);
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterGauss(src, dest, maskSize, destROIOffset);
	timeFlag("Call to ippiFilterGauss_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPFilterHipassWorker<uChar,3>
template <> QVIPPFilterHipassWorker<uChar,3>::QVIPPFilterHipassWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	ADD_IPPI_MASK_SIZE_PROPERTY(maskSize);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPFilterHipassWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	GET_IPPI_MASK_SIZE_PROPERTY(maskSize);
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterHipass(src, dest, maskSize, destROIOffset);
	timeFlag("Call to ippiFilterHipass_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPAddWorker<uChar,1>
template <> QVIPPAddWorker<uChar,1>::QVIPPAddWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src1", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("src2", inputFlag|outputFlag);
	addProperty< int >("scale", inputFlag, 1, "", 0, 16);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPAddWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src1 = getPropertyValue< QVImage<uChar,1> >("src1");
	const QVImage<uChar,1> src2 = getPropertyValue< QVImage<uChar,1> >("src2");
	const int scale = getPropertyValue< int >("scale");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,1> dest(src1.getCols() + destROIOffset.x(), src1.getRows() + destROIOffset.y());

	Add(src1, src2, dest, scale, destROIOffset);
	timeFlag("Call to ippiAdd_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPAddWorker<uChar,3>
template <> QVIPPAddWorker<uChar,3>::QVIPPAddWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src1", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("src2", inputFlag|outputFlag);
	addProperty< int >("scale", inputFlag, 1, "", 0, 16);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPAddWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src1 = getPropertyValue< QVImage<uChar,3> >("src1");
	const QVImage<uChar,3> src2 = getPropertyValue< QVImage<uChar,3> >("src2");
	const int scale = getPropertyValue< int >("scale");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src1.getCols() + destROIOffset.x(), src1.getRows() + destROIOffset.y());

	Add(src1, src2, dest, scale, destROIOffset);
	timeFlag("Call to ippiAdd_8u_C3R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPAddCWorker<uChar, 1>
template <> QVIPPAddCWorker<uChar,1>::QVIPPAddCWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< int >("summand for channel 1", inputFlag, 128, "", 0, 255);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));

	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	}

template <> void QVIPPAddCWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int channel1Summand = getPropertyValue< int >("summand for channel 1");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	AddC(src, channel1Summand, dest, 1, destROIOffset);
	timeFlag("Call to ippiAddC_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPAddCWorker<uChar, 3>
template <> QVIPPAddCWorker<uChar,3>::QVIPPAddCWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< int >("summand for channel 1", inputFlag, 128, "", 0, 255);
	addProperty< int >("summand for channel 2", inputFlag, 128, "", 0, 255);
	addProperty< int >("summand for channel 3", inputFlag, 128, "", 0, 255);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));

	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	}

template <> void QVIPPAddCWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int channel1Summand = getPropertyValue< int >("summand for channel 1");
	const int channel2Summand = getPropertyValue< int >("summand for channel 2");
	const int channel3Summand = getPropertyValue< int >("summand for channel 3");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	const uChar channelSummands[3] = {channel1Summand, channel2Summand, channel3Summand};
	AddC(src, channelSummands, dest, 1, destROIOffset);
	timeFlag("Call to ippiAddC_8u_C3R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// QVIPPAbsDiffWorker<uChar,1>
template <> QVIPPAbsDiffWorker<uChar,1>::QVIPPAbsDiffWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src1", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("src2", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void QVIPPAbsDiffWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src1 = getPropertyValue< QVImage<uChar,1> >("src1");
	const QVImage<uChar,1> src2 = getPropertyValue< QVImage<uChar,1> >("src2");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,1> dest(src1.getCols() + destROIOffset.x(), src1.getRows() + destROIOffset.y());

	AbsDiff(src1, src2, dest, destROIOffset);
	timeFlag("Call to ippiAdbsDiff_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}


// QVIPPCompareCWorker<uChar, 1>
template <> QVIPPCompareCWorker<uChar,1>::QVIPPCompareCWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	ADD_IPP_CMP_OP_PROPERTY(operation);
	addProperty< int >("operand for channel 1", inputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));

	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	}

template <> void QVIPPCompareCWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	GET_IPP_CMP_OP_PROPERTY(operation);
	const int channel1Operand = getPropertyValue< int >("operand for channel 1");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	CompareC(src, channel1Operand, dest, operation, destROIOffset);
	timeFlag("Call to ippiCompareC_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

const QStringList getQVIPPWorkersNames()
	{
	QStringList result;

	result.append("QVIPPCopyP3C3Worker<uChar>");
	result.append("QVIPPCopyC3P3Worker<uChar>");
	result.append("QVIPPErode3x3Worker<uChar,1>");
	result.append("QVIPPComputeThreshold_OtsuWorker<uChar,1>");
	result.append("QVIPPAddRandUniform_DirectWorker<uChar,1>");
	result.append("QVIPPFilterBoxWorker<uChar,1>");
	result.append("QVIPPFilterBoxWorker<uChar,3>");
	result.append("QVIPPFilterMedianWorker<uChar,1>");
	result.append("QVIPPFilterMedianWorker<uChar,3>");
	result.append("QVIPPFilterMaxWorker<uChar,1>");
	result.append("QVIPPFilterMaxWorker<uChar,1>");
	result.append("QVIPPFilterMinWorker<uChar,1>");
	result.append("QVIPPFilterMinWorker<uChar,3>");
	result.append("QVIPPFilterGaussWorker<uChar,1>");
	result.append("QVIPPFilterHipassWorker<uChar,1>");
	result.append("QVIPPFilterGaussWorker<uChar,3>");
	result.append("QVIPPFilterHipassWorker<uChar,3>");
	result.append("QVIPPAddWorker<uChar,1>");
	result.append("QVIPPAddCWorker<uChar, 1>");
	result.append("QVIPPAddCWorker<uChar, 3>");
	result.append("QVIPPAbsDiffWorker<uChar, 1>");
	result.append("QVIPPCompareCWorker<uChar, 1>");

	return result;
	}

QVPropertyContainer *newQVIPPWorker(QString type, QString name)
	{
	if (type == "QVIPPCopyP3C3Worker<uChar>")			return new QVIPPCopyP3C3Worker<uChar>(name);
	else if (type == "QVIPPCopyC3P3Worker<uChar>")			return new QVIPPCopyC3P3Worker<uChar>(name);
	else if (type == "QVIPPComputeThreshold_OtsuWorker<uChar,1>")	return new QVIPPComputeThreshold_OtsuWorker<uChar,1>(name);
	else if (type == "QVIPPAddRandUniform_DirectWorker<uChar,1>")	return new QVIPPAddRandUniform_DirectWorker<uChar,1>(name);
	else if (type == "QVIPPFilterBoxWorker<uChar,1>")		return new QVIPPFilterBoxWorker<uChar,1>(name);
	else if (type == "QVIPPFilterBoxWorker<uChar,3>")		return new QVIPPFilterBoxWorker<uChar,3>(name);
	else if (type == "QVIPPFilterMedianWorker<uChar,1>")		return new QVIPPFilterMedianWorker<uChar,1>(name);
	else if (type == "QVIPPFilterMedianWorker<uChar,3>")		return new QVIPPFilterMedianWorker<uChar,3>(name);
	else if (type == "QVIPPFilterMaxWorker<uChar,1>")		return new QVIPPFilterMaxWorker<uChar,1>(name);
	else if (type == "QVIPPFilterMaxWorker<uChar,3>")		return new QVIPPFilterMaxWorker<uChar,3>(name);
	else if (type == "QVIPPFilterMinWorker<uChar,1>")		return new QVIPPFilterMinWorker<uChar,1>(name);
	else if (type == "QVIPPFilterMinWorker<uChar,3>")		return new QVIPPFilterMinWorker<uChar,3>(name);
	else if (type == "QVIPPFilterGaussWorker<uChar,1>")		return new QVIPPFilterGaussWorker<uChar,1>(name);
	else if (type == "QVIPPFilterGaussWorker<uChar,3>")		return new QVIPPFilterGaussWorker<uChar,3>(name);
	else if (type == "QVIPPFilterHipassWorker<uChar,1>")		return new QVIPPFilterHipassWorker<uChar,1>(name);
	else if (type == "QVIPPFilterHipassWorker<uChar,3>")		return new QVIPPFilterHipassWorker<uChar,3>(name);
	else if (type == "QVIPPAddWorker<uChar,1>")	 		return new QVIPPAddWorker<uChar,1>(name);
	else if (type == "QVIPPAddCWorker<uChar, 1>")	 		return new QVIPPAddCWorker<uChar, 1>(name);
	else if (type == "QVIPPAddCWorker<uChar, 3>")	 		return new QVIPPAddCWorker<uChar, 3>(name);
	else if (type == "QVIPPAbsDiffWorker<uChar, 1>")	 	return new QVIPPAbsDiffWorker<uChar, 1>(name);
	else if (type == "QVIPPCompareCWorker<uChar, 1>")		return new QVIPPCompareCWorker<uChar, 1>(name);
	else return NULL;
	}

char *QVIPPWorkerName(QVPropertyContainer *cont)
	{
	if(dynamic_cast<QVIPPCopyP3C3Worker<uChar>*>(cont) != NULL) return "QVIPPCopyP3C3Worker<uChar>";
	else if(dynamic_cast<QVIPPCopyC3P3Worker<uChar>*>(cont) != NULL) return "QVIPPCopyC3P3Worker<uChar>";
	else if(dynamic_cast<QVIPPComputeThreshold_OtsuWorker<uChar,1>*>(cont) != NULL) return "QVIPPComputeThreshold_OtsuWorker<uChar,1>";
	else if(dynamic_cast<QVIPPAddRandUniform_DirectWorker<uChar,1>*>(cont) != NULL) return "QVIPPAddRandUniform_DirectWorker<uChar,1>";
	else if(dynamic_cast<QVIPPFilterBoxWorker<uChar,1>*>(cont) != NULL) return "QVIPPFilterBoxWorker<uChar,1>";
	else if(dynamic_cast<QVIPPFilterBoxWorker<uChar,3>*>(cont) != NULL) return "QVIPPFilterBoxWorker<uChar,3>";
	else if(dynamic_cast<QVIPPFilterMedianWorker<uChar,1>*>(cont) != NULL) return "QVIPPFilterMedianWorker<uChar,1>";
	else if(dynamic_cast<QVIPPFilterMedianWorker<uChar,3>*>(cont) != NULL) return "QVIPPFilterMedianWorker<uChar,3>";
	else if(dynamic_cast<QVIPPFilterMaxWorker<uChar,1>*>(cont) != NULL) return "QVIPPFilterMaxWorker<uChar,1>";
	else if(dynamic_cast<QVIPPFilterMedianWorker<uChar,3>*>(cont) != NULL) return "QVIPPFilterMedianWorker<uChar,3>";
	else if(dynamic_cast<QVIPPFilterMaxWorker<uChar,1>*>(cont) != NULL) return "QVIPPFilterMaxWorker<uChar,1>";
	else if(dynamic_cast<QVIPPFilterMaxWorker<uChar,3>*>(cont) != NULL) return "QVIPPFilterMaxWorker<uChar,3>";
	else if(dynamic_cast<QVIPPFilterMinWorker<uChar,1>*>(cont) != NULL) return "QVIPPFilterMinWorker<uChar,1>";
	else if(dynamic_cast<QVIPPFilterMinWorker<uChar,3>*>(cont) != NULL) return "QVIPPFilterMinWorker<uChar,3>";
	else if(dynamic_cast<QVIPPFilterGaussWorker<uChar,1>*>(cont) != NULL) return "QVIPPFilterGaussWorker<uChar,1>";
	else if(dynamic_cast<QVIPPFilterGaussWorker<uChar,3>*>(cont) != NULL) return "QVIPPFilterGaussWorker<uChar,3>";
	else if(dynamic_cast<QVIPPFilterHipassWorker<uChar,1>*>(cont) != NULL) return "QVIPPFilterHipassWorker<uChar,1>";
	else if(dynamic_cast<QVIPPFilterHipassWorker<uChar,3>*>(cont) != NULL) return "QVIPPFilterHipassWorker<uChar,3>";
	else if(dynamic_cast<QVIPPAddWorker<uChar,1>*>(cont) != NULL) return "QVIPPAddWorker<uChar,1>";
	else if(dynamic_cast<QVIPPAddCWorker<uChar, 1>*>(cont) != NULL) return "QVIPPAddCWorker<uChar, 1>";
	else if(dynamic_cast<QVIPPAddCWorker<uChar, 3>*>(cont) != NULL) return "QVIPPAddCWorker<uChar, 3>";
	else if(dynamic_cast<QVIPPAbsDiffWorker<uChar, 1>*>(cont) != NULL) return "QVIPPAbsDiffWorker<uChar, 1>";
	else if(dynamic_cast<QVIPPCompareCWorker<uChar, 1>*>(cont) != NULL) return "QVIPPCompareCWorker<uChar, 1>";
	else return NULL;
	}

